import json
from mlagents_envs.side_channel.side_channel import (
    SideChannel,
    IncomingMessage,
    OutgoingMessage,
)
import uuid

from EnvironmentData import EnvironmentData

"""
class that processes information from SideChannel EnvChannel
"""

class EnvChannel(SideChannel):

    def __init__(self) -> None:
        super().__init__(uuid.UUID("621f0a70-4f87-11ea-a6bf-784f4387d1f7")) #must coincide with ChannelId set in c#
        self.EnvData = EnvironmentData() # object that stores info on environment

    def on_message_received(self, msg: IncomingMessage) -> None:
        # We simply read a string from the message and print it.
        data = msg.read_string()
        environment_info = json.loads(data)

        self.EnvData.matrix = environment_info["matrixLayout"]
        self.EnvData.rotation = environment_info["rotationRobot"]
        self.EnvData.action_size = environment_info["sizeOfAction"]
        self.EnvData.observation_size = environment_info["sizeOfObservation"]
        self.EnvData.number_of_robots = environment_info["numberOfRobotsInEnv"]
        self.EnvData.number_of_current_tasks = environment_info["numberOfCurrentTasks"]
        self.EnvData.random_tasks = environment_info["tasksGeneratedRandomly"]
        self.EnvData.instance_ids = environment_info["instanceIDSRobots"]

        self.EnvData.set_initial_nodes_dictionary(environment_info["initialNodesOfRobots"])
        self.EnvData.set_tasks_to_be_assigned(environment_info["currentTasks"])
        self.EnvData.set_assigned_tasks()

    def send_string(self, data: str) -> None:
        # Add the string to an OutgoingMessage
        msg = OutgoingMessage()
        msg.write_string(data)
        # We call this method to queue the data we want to send
        super().queue_message_to_send(msg)