# Task-Allocation and Prioritized pathplanning (TA-Prioritized)

The Task-Allocation and Prioritized pathplanning (TA-Prioritized) algorithm is a centralized algorithm for the offline variant of the MAPD problem.

In the offline variant of the MAPD problem the following is supported:
- Using tasks from a taskslist with releaseTimes 
- Tested from 1 agent up untill 50 agents (significant lag for larger amount of agents)

Not supported:
- Randomly generated tasks (randomTasks = True)

This algorithm was added by Hicham Azmani. The corresponding Bachelor thesis has been added in the repository and can be found in [paper](/Algorithms/Offline_Centralized/Algorithm_TA_Prioritized/paper).

## Running the algorithm

1. If not yet done, install the virtual python environment
    - 'pipenv install'
2. Setup the LKH-3 TSP solver (see "LKH-3 TSP Solver" section)
3. Setup lkh.config (see below)
4. Run the "centralized_decisionmaker_shortest_path.py" file
4. Start the game inside the Unity editor

## Working of the algorithm

This implementation is a modified version of the implementation available at: https://github.com/Pieter-Cawood/M-TA-Prioritized-MAPD and  based on the "Task and Path Planning for Multi-Agent Pickup and Delivery" paper (http://idm-lab.org/bib/abstracts/papers/aamas19a.pdf).
An explanation of the algorithm is available at:
    - Section 3 and 4 of the "Task and Path Planning for Multi-Agent Pickup and Delivery" paper
    - Section 2.2.2 of the paper provided in the "paper" 
    
General working of the algorithm:
    - The algorithm uses the LKH-3 TSP Solver to generate task sequences for the agents:
        * Each agent gets an ordered list of tasks (or no tasks at all)
        * The agent needs to execute these tasks in the provided order
    - The algorithm plans 1 path for each agent to execute its whole task set:
        * Each agent gets one path that allows it to execute all its tasks in the correct order
        => In order to minimize the amount of timesteps needed to solve the problem the algorithm uses the Prioritized Pathplanning method to always plan longer paths first
        => This is done to avoid making the longer paths even longer by waiting to avoid collisions (see papers for a more detailed explanation)
    - Once all the agents have their path they start following that path
        * In each timestep each agent moves one step following its path

## (IMPORTANT) LKH-3 TSP Solver

- This algorithm uses a modified version of the original LKH-3 TSP solver (original: http://akira.ruc.dk/~keld/research/LKH-3/) to generate task sequences. In the [LKH3](/Algorithms/Offline_Centralized/Algorithm_TA_Prioritized/LKH3) directory you can find
    - LKH-3.exe: executable for Windows
    - LKH_linux: executable for Linux
    - LKH_mac: executable for mac
    - LKHWin-3.0.6-WinChanges.zip: visual studio project for Windows
    - SRC.zip: software for Linux/mac
- By default the algorithm computes new task sequences using the LKH-3 solver every time the algorithm is ran
- The algorithm supports using pre-computed task sequences (as this process can be slow for larger task sets) 

### Setup of LKH-3 TSP Solver

- The source code (and an executable for Linux, mac and Windows) are provided in the LKH-3 directory
- If the provided executable is not supported by your machine (error when trying to run the LKH-3 executable) you can compile a new executable on a Unix/Linux machine using the following commands:
    * cd LKH3
    * make
- On Windows machines you need to create an .exe file using the visual studio project provided in the LKH3/SRC directory

### Running the algorithm without using pre-computed task sequences

Although possible, it is recommended to not use use pre-computed task sequences. Time-outs may occur.

If you want the algorithm to compute task sequences and then pathplanning in one go you need to configure the lkh.conf file and run the main file.
- Set the lkh.conf file as follows and do not use any quotation marks, e.g. path=/User/LKH-3.exe
    * tourFile = None
    * path = specify path to LKH3 solver (can found in directory LKH3, Linux: LKH_linux, Windows: LKH-3.exe, Mac: LKH_mac)
    * lkhMaxTime = The maximum time (in seconds) that should be used by the LKH-3 TSP solver to compute task sequences (default: 500)
- Run the main file centralized_decisionmaker_shortest_path.py directly.

### Running the algorithm using pre-computed task sequences
If you want to run the algorithm using pre-computed task sequences, there are two options: either you have already a tour file containing the task sequences or you still need to generate it.

#### You already have the task sequences saved in a tour file
- Set the lkh.conf file as follows and do not use any quotation marks, e.g. path=/User/LKH-3.exe
    - tourFile = path to tour file computed in previous step.
    - path = path to LKH3 solver (Linux: LKH_linux, Windows: LKH-3.exe, Mac: LKH_mac)
    - lkhMaxTime = The maximum time (in seconds) that should be used by the LKH-3 TSP solver to compute task sequences (default: 500)
- Run the main file centralized_decisionmaker_shortest_path.py

#### You still need to compute the task sequences

First you need to generate the tour file. 
- Set the lkh.conf file as follows and do not use any quotation marks, e.g. path=/User/LKH-3.exe
    - tourFile = None.
    - path = specify path to LKH3 solver (can found in directory LKH3, Linux: LKH_linux, Windows: LKH-3.exe, Mac: LKH_mac)
    - lkhMaxTime = The maximum time (in seconds) that should be used by the LKH-3 TSP solver to compute task sequences (default: 500)
- Generate the .tour file using generate_tour_file.py:
    - Run generate_tour_file.py, a tour file will be generated and added in LKH3/TOUR. Each tour file created will have a unique name containing the day and hour.

Then you can follow the same steps as in the previous section:
- Set the lkh.conf file as follows and do not use any quotation marks, e.g. path=/User/LKH-3.exe
    - tourFile = path to tour file computed in previous step.
    - path = path to LKH3 solver (Linux: LKH_linux, Windows: LKH-3.exe, Mac: LKH_mac)
    - lkhMaxTime = The maximum time (in seconds) that should be used by the LKH-3 TSP solver to compute task sequences (default: 500)
- Run the main file centralized_decisionmaker_shortest_path.py

## Overview of the provided files

Main file:
- centralized_decisionmaker_shortest_path.py: Main file that runs the algorithm

Task Allocation and Prioritized Pathplanning:
- ta_agent.py: Agent that are executing tasks
- ta_system.py: System allocates tasks and plans paths for agents
- ta_task.py: Pickup and delivery tasks
- ta_world.py: Class that reads and stores the warehouse environment

A-star:
- ta_astar.py: A-Star algorithm
- ta_state.py: Contains Location and state classes used for the path planning

LKH-3:
- lkh.py: Class that computes task sequences using the LKH-3 TSP solver
- lkh.conf: Configuration file for the LKH-3 TSP solver