import os
import json
import numpy as np

from lkh import Agent, LKH
from ta_world import Location
from ta_task import Task

path_to_PickUpandDelivery = "../../../PickUpandDelivery/"
input_output = path_to_PickUpandDelivery + "InputOutput.txt"
with open(input_output) as input_output_file:
    input_output_json = json.load(input_output_file)

problemFile = path_to_PickUpandDelivery + input_output_json["ConfigFile"]
tasksFile = path_to_PickUpandDelivery + input_output_json["TasksFile"]

environment = None
agentsList = []
tasksList = []

with open(problemFile) as envFile:
    envData = json.load(envFile)

    environment = np.array(envData["Matrix"])

    agents = envData["InitialNodes"]

    for a in agents:
        col = a["GridPosition"]["Column"]
        row = a["GridPosition"]["Row"]
        rot = a["Degrees"]

        loc = Location(col, row, rot)

        agentsList.append(Agent(loc))

with open(tasksFile) as tasksFile:

    tasksData = json.load(tasksFile)

    taskID = 1

    for task in tasksData:

        pickup = Location(task['PickUp']["Column"],
                          task['PickUp']["Row"],
                          0)

        delivery = Location(task['Delivery']["Column"],
                            task['Delivery']["Row"],
                            0)

        releaseTime = task['ReleaseTime']

        tasksList.append(Task(taskID, releaseTime, pickup, delivery, None))

        taskID += 1

lkh = LKH(environment, tasksList, agentsList)

lkhFile = lkh.get_task_sequences()

print(f"Computed task sequences saved: {lkhFile}")