"""
Created on Sat Apr 18 12:51:32 2020

@author: Pieter Cawood
"""
from queue import Queue
import os

CURRENT_TASK_ID = 0


class Task(object):
    def __init__(self, task_id, release_time, endpoint_pickup, endpoint_delivery, world):
        self.id = task_id
        self.assigned_agent = None
        self.release_time = release_time
        self.pickup_endpoint = endpoint_pickup
        self.delivery_endpoint = endpoint_delivery

        self.delivered = None

        self.pickup_id = None
        self.delivery_id = None

    def service_time(self):
        if self.delivered is not None:
            return (self.delivered - self.release_time)
        else:
            return None


# With key of task_id to pair with TaskSequence
class TaskDict(dict):
    def __init__(self, taskList, world):
        self.load_tasks(taskList, world)

    def load_tasks(self, taskList, world):
        
        for t in taskList:
            self[t.id] = t


        # task_id = 0
        # for t in taskList:
        #     task_releaseTime = t[0]
        #     task_pickup = t[1]
        #     task_delivery = t[2]

        #     self[task_id] = Task(task_id, task_releaseTime, task_pickup, task_delivery, world)
        #     task_id += 1
        #print(f"(2/3) Tasks loaded")


# With key of seq id (max is agent count) to load all tasks
# Seq_id is initially agent listed in tour file
class TourDict(dict):
    def __init__(self, tour_file_path, agent_count, tasks_dict):
        self.load_file(tour_file_path, agent_count, tasks_dict)

    def load_file(self, tour_file_path, agentsList, tasks_dict):
        agent_count = len(agentsList)
        file = open(tour_file_path, "r")
        if not file.mode == 'r':
            print("Could not open " + tour_file_path)
        else:
            tour_data = file.readlines()
            tour_section = False
            agent_sequence = None
            agent_number = 1
            ez = 0
            for line in tour_data:
                if "-1" in line:  # end of data
                    if agent_sequence.qsize():
                        agent = agentsList[agent_number-1]
                        a_id = agent.id
                        self[a_id] = agent_sequence
                        tour_section = False
                if tour_section:
                    line_val = int(line)
                    if line_val <= agent_count:  # line listed an agent number
                        # print(f"Reading sequence of agent: {ez} / {agent_count}")
                        if agent_sequence.qsize():  # Store previous agent queue
                            # print(f"Saved sequence for agent {agent_number} with size {agent_sequence.qsize()}")
                            agent = agentsList[agent_number-1]
                            a_id = agent.id
                            self[a_id] = agent_sequence
                            agent_sequence = Queue()  # Create new agent queue
                            agent_number = line_val
                        ez += 1
                    else:
                        # Line listed as task number, subtract agent count
                        task_id = line_val - agent_count - 1
                        agent_sequence.put(tasks_dict[task_id])
                        # Set initial agent to tsp seq
                        tasks_dict[task_id].seq_id = agent_number
                if "TOUR_SECTION" in line:  # start of data next line
                    tour_section = True
                    agent_sequence = Queue()
        print(f"(3/3) Sequences loaded")
