"""
Created on Sat Apr 18 10:49:17 2020

@author: Pieter Cawood

"""
from enum import Enum
from ta_agent import *
from ta_state import *


class MAPNODETYPES(Enum):
    PARKING = 0
    PATH = 1
    WALL = 2
    TASK_ENDPOINT = 3


# Loads map. parking and endpoints are numbered in sequence loaded.
# Task endpoints numbered from agent_number + 1
class World(dict):
    def __init__(self, env, agentsList):
        self.agents = dict()
        self.endpoints = dict()
        self.parking_locations = dict()
        self.width = 0  # Map width
        self.height = 0  # Map height
        self.load_map(env)
        self.load_agents(env, agentsList)

    def load_agents(self, env, agentsList):
        for agent in agentsList:
            agent_col = agent.parking_location.col
            agent_row = agent.parking_location.row
            if env[agent_row][agent_col] != 2:
                raise Exception('Agent {} has an invalid parking location: {}'.format(agent.id, agent.parking_location))
            self.agents[agent.id] = agent
            agent.path[0] = agent.parking_location
            self.parking_locations[agent.id] = agent.parking_location

    def load_map(self, env):
        parking_count = 1
        endpoint_count = 0
        for row in range(len(env)):
            for col in range(len(env[0])):
                node = None
                if env[row][col] == 0:
                    # Node is a path
                    node = MAPNODETYPES.PATH
                elif env[row][col] == 2:
                    # Node is a parking location (non-task endpoint)
                    node = MAPNODETYPES.PARKING
                elif env[row][col] == 1:
                    # Node is a wall
                    node = MAPNODETYPES.WALL
                elif env[row][col] == 3:
                    # Node is a pickup/delivery location (task endpoint)
                    node = MAPNODETYPES.TASK_ENDPOINT
                    self.endpoints[endpoint_count] = Location(col, row, 0)
                    endpoint_count += 1
                if node is not None:
                    # Add node to this collection
                    self[col, row] = node
        self.width = len(env[0])
        self.height = len(env)
        print(f"(1/3) Map loaded")
        print(f"Created {parking_count} agents")
